﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel.Channels;
using System.Xml.Linq;
using System.Xml;
using System.IO;
using System.ServiceModel.Web;

namespace MVeldhuizen.DevTools.ServiceBehaviors
{
    /// <summary>
    /// This service generates clientaccesspolicy.xml and crossdomain.xml configuration files
    /// on the fly. 
    ///
    /// Intended for use together with WCF Service Host. See my blog article for more details:
    /// http://thorarin.net/blog/post.aspx?id=70618cd9-eb8c-4d9f-8e49-9bdde3b2eba6
    /// </summary>
    public class CrossDomainPolicyService : ICrossDomainPolicyService
    {
        private static readonly XmlWriterSettings _xmlWriterSettings;

        static CrossDomainPolicyService()
        {
            _xmlWriterSettings = new XmlWriterSettings()
            {
                Encoding = Encoding.UTF8,
                Indent = true
            };
        }

        #region ICrossDomainPolicyService Members

        public Stream GetClientAccessPolicyFile()
        {
            XDocument doc = new XDocument(
                new XElement("access-policy",
                    new XElement("cross-domain-access",
                        new XElement("policy",
                            new XElement("allow-from", new XAttribute("http-request-headers", "SOAPAction"),
                                new XElement("domain", new XAttribute("uri", "*"))),
                            new XElement("grant-to",
                                new XElement("resource", new XAttribute("path", "/"), new XAttribute("include-subpaths", "true")))))));

            return CreateOutputStream(doc);
        }

        public Stream GetCrossDomainPolicyFile()
        {
            XDocument doc = new XDocument(
                new XDocumentType("cross-domain-policy", null, "http://www.adobe.com/xml/dtds/cross-domain-policy.dtd", null),
                new XElement("cross-domain-policy",
                    new XElement("allow-access-from", new XAttribute("domain", "*")),
                    new XElement("allow-http-request-headers-from", new XAttribute("domain", "*"), new XAttribute("headers", "SOAPAction"))));

            return CreateOutputStream(doc);
        }

        #endregion

        private Stream CreateOutputStream(XDocument doc)
        {
            WebOperationContext.Current.OutgoingResponse.ContentType = "application/xml; charset=" + _xmlWriterSettings.Encoding.WebName;

            MemoryStream ms = new MemoryStream();
            using (XmlWriter writer = XmlWriter.Create(ms, _xmlWriterSettings))
            {
                doc.Save(writer);
            }

            ms.Position = 0;
            return ms;
        }
    }
}
